/* HexStrtoDecStr.cp */

/*
	written by
	Dr. Robert M. Delaney
	Emeritus Professor of Physics
	Saint Louis University
	6/23/05
*/

#include "HexStrtoDecStr.h"
#include <string.h>
#include <stdlib.h>

/*
	 Calling convention:
	 Create hexStr as an unsigned hex number C string.
	 then
	 char	*decStr;
	 decStr = HexStrToDecStr(hexStr);
	 // do whatever
	 free(decStr);
	 return;
	 
	 hexStr will be returned in a modified state, no longer a string of hex characters
*/

char* HexStrtoDecStr(char* hexStr)
{
	long		iChar, jByte, k, length, numBytes;
	short		carry, temp, temp1, ten=10, sixteen=16;
	char*		decStr;
	char 		*dlookup = "0123456789", ch;
	
	length = strlen(hexStr);
	
	if(length==0)
	{
		decStr = (char*)malloc(sizeof(char));
		decStr[0] = 0;
		return decStr;
	}
		
	// on average there are 3.32 bits per decimal digit or 0.83 hex digits per decimal digit
	// so, on average we have 1.20 decimal digits per one hex digit; we'll use 1.3
	
	decStr = (char*)malloc((1.3*length+4)*sizeof(char)); // don't free in this method!
	
	// we don't collapse the string, but we do have to get the numerical bytes for base 16
	for(k=0;k<length;k++)
	{
		ch = hexStr[k];
		if(!(ch>='0' && ch<='9') && !((ch>='A' && ch<='F') || (ch>='a' && ch<='f')))
		{
			decStr[0] = 0;
			return decStr;
		}
		
		if(ch>='0' && ch<='9')
			hexStr[k] = (ch - '0');
		
		if(ch>='A' && ch<='F')
			hexStr[k] = ten + (ch - 'A');
		
		if(ch>='a' && ch<='f')
			hexStr[k] = ten + (ch - 'a');
	}
	
	numBytes = length;
	iChar = 0;
	jByte = 0;
	
	while(numBytes>0)
	{
		temp = hexStr[jByte];
		for(k=jByte;k<length;k++)
		{
			// divide temp by 10 and get carry (0 to 9)
			temp1 = temp/ten;
			carry = temp - ten*temp1;
			hexStr[k] = temp1;  // new byte value after dividing by ten
			if(k==length-1)
			{
				decStr[iChar] = dlookup[carry];
				iChar++;
			}
			else
			{
				temp = sixteen*carry + hexStr[k+1];
			}
		}
		if(hexStr[jByte]==0)
		{
			jByte++;
			numBytes--;
		}
	}
	
	decStr[iChar] = 0;
	
	// reverse the chars
	length = strlen(decStr);
	for(k=0;k<length/2;k++)
	{
		ch = decStr[k];
		decStr[k] = decStr[length-1-k];
		decStr[length-1-k] = ch;
	}
	
	return decStr;
	
}/* HexStrtoDecStr */

